//*****************************************************************************
//*****************************************************************************
//  FILENAME:   amn1x11x.c
//  Version: 1.0, 2007/02/06
//  Generated by Andy Kiser
//
//  DESCRIPTION:  functions for the amn1x11x motion detector
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "psocgpioint.h"
#include "amn1x11x.h"
#include "feature.h"
#include "psocapi.h"

//----------------------------------------
// this module's global variables
extern WORD MotionCounter; // modified in GPIO interrupt service routine

WORD		MotionCounts;


//----------------------------------------
// type definitions

typedef enum 
{	
	eReset,
	eIdle,
	eReadCounter
} tFSM_Measure;

//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

BOOL			NewSample = FALSE;

//-----------------------------------------------------------------------------
// Initializes the amn1x11x
//-----------------------------------------------------------------------------
void amn1x11x_Start()
{
	// turn on power to the sensor
	ON_nOFF_Data_ADDR |= ON_nOFF_MASK;
	
	// initialize motion counter
	MotionCounter = 0;

	// Enable GPIO interrupt for MOTION input
	MOTION_IntEn_ADDR |= MOTION_MASK;
	INT_MSK0 |= INT_MSK0_GPIO;

	// initialize state machine
	FSM = eReset;
}
//-----------------------------------------------------------------------------
// Prepare amn1x11x for Suspend
//-----------------------------------------------------------------------------
// TODO: implement wake on motion
void amn1x11x_Suspend()
{
	// disable GPIO interrupt
	INT_MSK0 &= ~INT_MSK0_GPIO;
	
	// turn off power for the sensor
	ON_nOFF_Data_ADDR &= ~ON_nOFF_MASK;
}
//-----------------------------------------------------------------------------
// Resume amn1x11x from Suspend mode
//-----------------------------------------------------------------------------
void amn1x11x_Resume()
{
	amn1x11x_Start();
}

//-----------------------------------------------------------------------------
// Read ADC 
//-----------------------------------------------------------------------------
WORD amn1x11x_ReadCount (void)
{
	WORD cnt;
	
	M8C_DisableGInt;	// avoid updating of the counter during readout
	cnt = MotionCounter;
	MotionCounter = 0;
	M8C_EnableGInt;

	return cnt;
}


//-----------------------------------------------------------------------------
// void amn1x11x_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the amn1x11x sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void amn1x11x_FSM_Measure() 
{
	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{
		case eReset:
			Countdown_1ms = 1; // 1ms delay until first measurement finished
			FSM = eIdle;
			break;
			
		case eIdle: 
			if (Countdown_1ms == 0)
			{
	  		    Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
				FSM = eReadCounter;					  
			}
			break;			
			
		case eReadCounter:
			MotionCounts = amn1x11x_ReadCount();
			NewSample = TRUE;
			
			FSM = eIdle;
			break;			

	}
}

